import requests
import hashlib
import json

from ulib import typetools


##### Public constants #####
OK = "OK"
INFO = "INFO" # XXX: Don't use this status! This is only for compatibility with Juggler.
WARN = "WARN"
CRIT = "CRIT"

TO_GOLEM_STATUS_MAP = {
    OK:   "ok",
    INFO: "ok",
    WARN: "warning",
    CRIT: "critical",
}


##### Exceptions #####
class GolemApiError(Exception):
    def __init__(self, result):
        super(GolemApiError, self).__init__()
        self._result = result

    def get_result(self):
        return self._result

    def __str__(self):
        return "{status_code} {status_message}: {golem_response}".format(
            status_code=self._result.status_code,
            status_message=self._result.reason,
            golem_response=self._result.text,
        )

class GolemNoSmsRecepientsError(GolemApiError):
    pass

class GolemNoCurrentEventError(GolemApiError):
    pass


##### Public methods #####
def normalize_description(status, description):
    if not isinstance(description, dict):
        return {
            "status":      status,
            "description": description,
        }
    elif "status" not in description:
        description = dict(description)
        description["status"] = status
    return description

def make_description_hash(description):
    return typetools.object_hash(
        description,
        hasher=( lambda text: hashlib.md5(text.encode()).hexdigest() ),
    )


##### Public classes #####
class Golem:
    def __init__(self, url, **kwargs):
        self._url = url
        self._kwargs = kwargs


    ### Public ###

    def get_responsibles(self, host, handle="api/get_host_resp.sbml"):
        result = self._call_api(handle, { "host": host })
        if result.status_code != 200:
            raise GolemApiError(result)
        return list(filter(None, result.text.strip().split(",")))

    ###

    def send_sms_for_user(self, to, body, handle="api/sms/send.sbml"):
        if not isinstance(to, (list, tuple)):
            to = (to,)
        self._send_sms({ "resps": ",".join(to) }, body, handle)

    def send_sms_for_service(self, host, service, body, handle="api/sms/send.sbml"):
        self._send_sms({
                "object": host,
                "eventtype": service,
            }, body, handle)

    def send_event(self, host, service, status, description, description_hash=None, monitor=None, handle="api/events/submit.sbml"):
        description = normalize_description(status, description)
        if description_hash is None:
            description_hash = make_description_hash(description)

        attrs = {
            "object":    host,
            "eventtype": service,
            "status":    TO_GOLEM_STATUS_MAP[status],
            "json":      "true",
            "md":        description_hash,
            "info":      json.dumps(description),
        }
        if monitor is not None:
            attrs["monitor"] = monitor

        result = self._call_api(handle, attrs)

        if result.status_code == 200 and result.text.startswith("ok"):
            return
        elif result.text.startswith("error: no current event"):
            raise GolemNoCurrentEventError(result)
        else:
            raise GolemApiError(result)

    ### Private ###

    def _send_sms(self, criteria, body, handle):
        attrs = { "msg": body }
        attrs.update(criteria)
        result = self._call_api(handle, attrs)
        if result.status_code == 200 and result.text.startswith("ok"):
            return
        elif result.text.startswith("error: no valid users"):
            raise GolemNoSmsRecepientsError(result)
        else:
            raise GolemApiError(result)

    def _call_api(self, handle, attrs):
        return requests.post("{}/{}".format(self._url, handle), data=attrs, **self._kwargs)
